# -*- coding: utf-8 -*-

from django import template
import urllib
from django.core.urlresolvers import reverse

register = template.Library()
from django.conf import settings

def encoded_dict(in_dict):
    out_dict = {}
    for k, v in in_dict.iteritems():
        if isinstance(v, unicode):
            v = v.encode('utf8')
        elif isinstance(v, str):
            # Must be encoded in UTF-8
            v.decode('utf8')
        out_dict[k] = v
    return out_dict

@register.simple_tag(takes_context=True)
def facebook_feed_url(context, friend, link, picture, name, caption, description, action_name, action_link, redirect_uri):
    """
        Direct URL Example
        
        You can also bring up a Feed Dialog by explicitly directing people to the /dialog/feed endpoint:
        
        https://www.facebook.com/dialog/feed?
          app_id=458358780877780&
          link=https://developers.facebook.com/docs/reference/dialogs/&
          picture=http://fbrell.com/f8.jpg&
          name=Facebook%20Dialogs&
          caption=Reference%20Documentation&
          description=Using%20Dialogs%20to%20interact%20with%20users.&
          redirect_uri=https://mighty-lowlands-6381.herokuapp.com/
        
        Click here to try the url yourself. The user will see a dialog that looks like the following:
        
        If a person clicks "Share", the browser will redirect to
        
        https://mighty-lowlands-6381.herokuapp.com/?post_id=12345
        
        The published story will look like this:
        
        If the user clicks "Cancel", the browser will redirect to
        
        https://mighty-lowlands-6381.herokuapp.com/
    """
    
    facebook_app_id = settings.FACEBOOK_APP_ID
    
    param_dict = {
      'app_id': facebook_app_id,
      'link': link,
      'picture': picture,
      'name'  : name,
      'caption': caption,
      'description' : description,
      'redirect_uri': redirect_uri,      
      'to': friend,
      'display' : 'popup',
    }
    
    if action_name != '' and action_link != '':
        param_dict['actions']="[{name:'%s',link:'%s'}]" % (action_name,action_link)
    
    params = urllib.urlencode(encoded_dict(param_dict))
    
    
    return "https://www.facebook.com/dialog/feed?%s" % params
    
    
#     return "https://www.facebook.com/dialog/feed?app_id=%s&to=%s&actions=[{name:'%s',link:'%s'}]&link=%s&picture=%s&name=%s&caption=%s&description=%s&redirect_uri=%s&display=popup" % (
#                         facebook_app_id, friend, action_name, action_link, link, picture, name, caption, description, redirect_uri)
# from django_facebook_helper.templatetags.django_facebook_helper_tags import facebook_feed_url
# facebook_feed_url(226504147430937, 1273360443, None, None, "A Bear Named Winnie (TV)", None, None, None, None, "http://local.filmsobsession.com:8000/facebook-management/facebook-redirect/")
# https://www.facebook.com/dialog/feed?app_id=226504147430937&to=1273360443&actions=[{name:'Comprar%20Pelicula',link:'https://www.google.es'}]&link=http://apps.facebook.com/filmsobsession/movies/detail/a-20-millas-de-la-justicia-a-veinte-millas-de-la-justicia/&picture=http://fo.dev.binomia.es/media/images/movies/W/what_doesn_t_kill_you.jpg&name=A%2020%20millas%20de%20la%20justicia%20(A%20veinte%20millas%20de%20la%20justicia)&caption=filmsobsession.com&description=Una%20organizaci%C3%B3n%20mafiosa%20se%20dedica%20a%20introducir%20inmigrantes%20en%20EE.UU,%20burlando%20a%20la%20polic%C3%ADa%20fronteriza.%20Un%20d%C3%ADa%20dicha%20organizaci%C3%B3n%20comete%20un%20error%20y%20asesina%20a%20sangre%20fr%C3%ADa%20a%20uno%20de%20los%20polic%C3%ADas.%20Charles%20Bronson%20encarna%20a%20un%20duro%20y%20curtido%20polic%C3%ADa%20que%20remover%C3%A1%20cielo%20y%20tierra%20para%20vengar%20la%20muerte%20de%20su%20compa%C3%B1ero.%20&redirect_uri=http://local.filmsobsession.com:8000/facebook-management/facebook-redirect/&display=popup




@register.simple_tag(takes_context=True)
def facebook_request_app_install(context,redirect_uri):
    """
        Este tag genera el enlace para recomendar la app a un amigo 
        
        Direct URL Example
        
        This example uses the the raw URL to render a Request Dialog for your app.
        One key difference from using this method over the JavaScript SDK is the 
        requirement of the redirect_uri parameter.
        
        https://www.facebook.com/dialog/apprequests?
          app_id=APP_ID&
          message=Facebook%20Dialogs%20are%20so%20easy!&
          redirect_uri=http://www.example.com/response
        
        After the user follows the flow and sends the request the browser will redirect to
        
        http://example.com/response?request=REQUEST_ID&to=ARRAY_OF_USER_IDS
        
        If there are errors, the browser will redirect to
        
        http://example.com/response?error_code=ERROR_CODE&error_msg=ERROR_MSG
    """
    facebook_app_id = settings.FACEBOOK_APP_ID
    facebook_app_message = settings.FACEBOOK_APP_INSTALL_MESSAGE
    
    params = urllib.urlencode({
      'app_id': facebook_app_id, 
      'message': facebook_app_message, 
      'redirect_uri'  : redirect_uri,
    })
    
    return u"https://www.facebook.com/dialog/apprequests?%s" % params



# -*- encoding: utf-8 -*-

from django.contrib.auth.models import User
from models import UserProfile

import logging
import facebook

from django.utils.simplejson import JSONEncoder

logger = logging.getLogger(__name__)

class FacebookBackend:
    
    """
        Token based authentication
    """
    def authenticate(self, token=None):        
        
        # Create an instance of facebook API centered in the user node.
        graph = facebook.GraphAPI(token)

        # Just do one request 
        all_fb_info = graph.request(
                      'me',
                      args={
                            'fields':
                            'id,name,friends,email,first_name,username,last_name'
                            })



        # Ask fb the user info.
#         profile = graph.get_object("me")
#         logger.debug(profile)
        
        # Stores a json list of friends
        friends_json = None
        
        # try to get friends
        try:
#             friends = graph.get_connections("me", "friends") 
#             logger.debug(friends)
#             
#             friends_json = JSONEncoder().encode(friends["data"])
#             logger.debug(friends_json)
            
            friends_json = JSONEncoder().encode(all_fb_info["friends"]["data"])
            logger.debug(friends_json)
            
        except Exception as e:
            logger.error(e)
            
      
        #Obtains fb user id.
#         uid = profile["id"]
        uid = all_fb_info["id"]
            
        try:
            # try to get a user based on fb uid stored in db.
            userProfile = UserProfile.objects.get(uid=uid)
            logger.debug("Encontrado userProfile: %s" % userProfile.user.username)
            
            #Update token
            userProfile.access_token = token
            
            if friends_json != None:
                logger.debug("friends to save: %s" % friends_json) 
                userProfile.friends = friends_json
            
            """
            TODO: falta por actualizar la fecha de expiración del token.
            """
            
            userProfile.save()
        
            
            return userProfile.user
        
        except UserProfile.DoesNotExist:
            logger.debug("UserProfile doesn't exists")
            
            try:
                logger.debug("Creating User.")       
                user = User()
                
                user, created = User.objects.get_or_create(
                                   username=uid, 
                                   defaults={
                                     'first_name': all_fb_info['first_name'],
                                     'last_name' : all_fb_info['last_name'],
                                     'email'     : all_fb_info['email'],
                                   })
                if created:
                    user.set_unusable_password()
                    user.save()
                    
                    
            except Exception as e:
                logger.error(e)
                return
                
              
            try:
                logger.debug("Creating UserProfile.")
                userProfile = UserProfile()
                userProfile.user = user;
                userProfile.access_token = token
                userProfile.uid = uid
                
                if friends_json != None:
                    logger.debug("friends to save: %s" % friends_json) 
                    userProfile.friends = friends_json
                    
                                  
                userProfile.save()
                logger.debug("Creado userProfile: %s" % user)
                
            except Exception as e:
                logger.error(e)
                
                
                
                
            return user
          
        # Never reached.  
        return user
   
   
    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None
        


from facebook import parse_signed_request
from django.contrib import auth
from django.conf import settings

import logging


logger = logging.getLogger(__name__)


class SignedRequestMiddleware:
    
    def process_view(self, request, view_func, view_args, view_kwargs):
        try:
            #Get signed_request sent by facebook
            signed_request = request.POST["signed_request"]
            #Decode of singed request with the FACEBOOK_APP_SECRET
            parsed_request = parse_signed_request(signed_request, settings.FACEBOOK_APP_SECRET)
            
            if parsed_request is False:
                return
                      
            #Avoid CSRF comprobation.
            view_func.csrf_exempt = True
            
            logger.debug("signed_request: %s" % signed_request)
            logger.debug("parsed_request: %s" % parsed_request)
                    
            #Try to authenticate the user with the authbackends
            #Eventually the backend django_facebook_helper.FacebookBackend will authenticate this User
            user = auth.authenticate(token=parsed_request["oauth_token"])
            logger.debug("user: %s" % user)
                  
            
            
            if user is not None:
                if user.is_active:
                    auth.login(request, user)
                    # Redirect to a success page.
                else:
                    pass # Return a 'disabled account' error message
            else:
                # Return an 'invalid login' error message.
                pass
        except:
            pass




from django.conf.urls import patterns, include, url

urlpatterns = patterns('',
    # Examples:    
    url(r'^auth-redirect/$', 'django_facebook_helper.views.login_redirect', name='facebook-auth-redirect'),
    url(r'^facebook-redirect/$', 'django_facebook_helper.views.facebook_redirect', name='facebook-redirect'),
    url(r'^test/friends/$', 'django_facebook_helper.views.friends', name='facebook-test-friends'),
    url(r'^test/wallpost/$', 'django_facebook_helper.views.wall_post', name='facebook-test-wall-post'),
)



from django.db import models
from django.contrib.auth.models import User
from django.utils.simplejson import JSONDecoder


import logging
logger = logging.getLogger(__name__)



class UserProfile(models.Model):
    user = models.ForeignKey(User, unique=True)
    access_token = models.CharField(max_length=255, unique=True)
    expires = models.IntegerField(null=True)
    uid = models.BigIntegerField(unique=True, null=True)
    
    friends = models.TextField()

        
    class Meta:
        unique_together = ('user', 'uid')
        
    def __unicode__(self):
        return u'Session for %s (%s) ' % (self.user.username, self.user.email)
    
    def get_friends(self):
        """
        Gets a python structure contaning facebook friends.
        """
        try:
            return JSONDecoder().decode(self.friends)
        except:
            return None
            


User.profile = property(lambda u: UserProfile.objects.get_or_create(user=u)[0])